@@ -49,7 +49,7 @@ AudioManager::AudioManager ( PluginManager *sharedPluginManager ) :
 	oAudioPlugin(NULL), selectedAudioBackend(""), pluginManager(sharedPluginManager)
 {
 //	  string DesiredAudio = get_audioConfig(); //Looks for the audio selected in the user's config
-	string DesiredAudio = getSys()->config->getAudioBackendName();
+	string DesiredAudio = Config::getConfig()->getAudioBackendName();
 	set_audiobackend ( DesiredAudio );
 }
 
@@ -58,18 +58,6 @@ bool AudioManager::pluginLoaded() const
 	return oAudioPlugin != NULL;
 }
 
-void AudioManager::freeStreamPlugin ( AudioStream *audioStream )
-{
-	if ( pluginLoaded() )
-	{
-		oAudioPlugin->freeStream ( audioStream );
-	}
-	else
-	{
-		LOG ( LOG_ERROR, _ ( "No audio plugin loaded, can't free stream" ) );
-	}
-}
-
 AudioStream *AudioManager::createStreamPlugin ( AudioDecoder *decoder )
 {
 	if ( pluginLoaded() )
@@ -83,32 +71,6 @@ AudioStream *AudioManager::createStreamPlugin ( AudioDecoder *decoder )
 	}
 }
 
-void AudioManager::pauseStreamPlugin( AudioStream *audioStream )
-{
-	if ( pluginLoaded() )
-	{
-		oAudioPlugin->pauseStream ( audioStream );
-	}
-	else
-	{
-		LOG ( LOG_ERROR, _ ( "No audio plugin loaded, can't pause stream" ) );
-	}
-
-}
-
-void AudioManager::resumeStreamPlugin( AudioStream *audioStream )
-{
-	if ( pluginLoaded() )
-	{
-		oAudioPlugin->resumeStream ( audioStream );
-	}
-	else
-	{
-		LOG ( LOG_ERROR, _ ( "No audio plugin loaded, can't resume stream" ) );
-	}
-
-}
-
 bool AudioManager::isTimingAvailablePlugin() const
 {
 	if ( pluginLoaded() )
