@@ -251,12 +251,12 @@ public:
 	 * Converts data (which is in RGB format) to the format internally used by cairo.
 	 * This function new[]'s the returned value, which has to be freed by the caller.
 	 */
-	static uint8_t* convertBitmapToCairo(uint8_t* data, uint32_t width, uint32_t height, size_t* dataSize);
+	static uint8_t* convertBitmapToCairo(uint8_t* data, uint32_t width, uint32_t height, size_t* dataSize, size_t* stride);
 	/*
 	 * Converts data (which is in ARGB format) to the format internally used by cairo.
 	 * This function new[]'s the returned value, which has to be freed by the caller.
 	 */
-	static uint8_t* convertBitmapWithAlphaToCairo(uint8_t* inData, uint32_t width, uint32_t height, size_t* dataSize);
+	static uint8_t* convertBitmapWithAlphaToCairo(uint8_t* inData, uint32_t width, uint32_t height, size_t* dataSize, size_t* stride);
 };
 
 class CairoTokenRenderer : public CairoRenderer
