@@ -17,10 +17,13 @@
     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 **************************************************************************/
 
+#include <fstream>
 #include "logger.h"
+#include "threading.h"
 
-sem_t Log::mutex;
-bool Log::loggingInited = false;
+using namespace lightspark;
+
+static StaticMutex mutex;
 LOG_LEVEL Log::log_level=LOG_INFO;
 const char* Log::level_names[]={"ERROR", "INFO","NOT_IMPLEMENTED","CALLS","TRACE"};
 int Log::calls_indent = 0;
@@ -42,9 +45,8 @@ Log::~Log()
 {
 	if(valid)
 	{
-		sem_wait(&mutex);
-		std::cout << level_names[cur_level] << ": " << message.str();
-		sem_post(&mutex);
+		Mutex::Lock l(mutex);
+		std::cerr << level_names[cur_level] << ": " << message.str();
 	}
 }
 
@@ -53,12 +55,16 @@ std::ostream& Log::operator()()
 	return message;
 }
 
-void Log::initLogging(LOG_LEVEL l)
+void Log::print(const std::string& s)
 {
-	if(!loggingInited)
-	{
-		loggingInited=true;
-		sem_init(&mutex,0,1);
-		log_level=l;
-	}
+	Mutex::Lock l(mutex);
+	std::cout << s << std::endl;
+}
+
+void Log::redirect(std::string filename)
+{
+	Mutex::Lock l(mutex);
+	static std::ofstream file(filename);
+	std::cout.rdbuf(file.rdbuf());
+	std::cerr.rdbuf(file.rdbuf());
 }
