@@ -344,6 +344,7 @@ DefineSpriteTag::DefineSpriteTag(RECORDHEADER h, std::istream& in):DictionaryTag
 				empty=true;
 				break;
 			}
+			case SYMBOL_CLASS_TAG:
 			case ABC_TAG:
 			case CONTROL_TAG:
 				throw ParseException("Control tag inside a sprite. Should not happen.");
@@ -653,17 +654,27 @@ DefineBitsLosslessTag::DefineBitsLosslessTag(RECORDHEADER h, istream& in, int ve
 			inData[i] = 0xFF;
 	}
 
-	bitmapData->fromRGB(inData, BitmapWidth, BitmapHeight, true);
-	Bitmap::updatedData();
+	fromRGB(inData, BitmapWidth, BitmapHeight, true);
 }
 
 ASObject* DefineBitsLosslessTag::instance() const
 {
 	DefineBitsLosslessTag* ret=new DefineBitsLosslessTag(*this);
-	if(bindedTo)
+	//Flex imports bitmaps using BitmapAsset as the base class, which is derived from bitmap
+	//Also BitmapData is used in the wild though, so support both cases
+	if(bindedTo && bindedTo->isSubClass(Class<Bitmap>::getClass()))
+	{
+		ret->setClass(Class<BitmapData>::getClass());
+		Bitmap* bitmapRet=new Bitmap(_MR((BitmapData*)ret));
+		bitmapRet->setClass(bindedTo);
+		return bitmapRet;
+	}
+	else if(bindedTo && bindedTo->isSubClass(Class<BitmapData>::getClass()))
+	{
 		ret->setClass(bindedTo);
+	}
 	else
-		ret->setClass(Class<Bitmap>::getClass());
+		ret->setClass(Class<BitmapData>::getClass());
 	return ret;
 }
 
@@ -1351,17 +1362,26 @@ MetadataTag::MetadataTag(RECORDHEADER h, std::istream& in):Tag(h)
 {
 	LOG(LOG_TRACE,_("MetadataTag Tag"));
 	in >> XmlString;
-
 	string XmlStringStd = XmlString;
-	xmlpp::TextReader xml((const unsigned char*)XmlStringStd.c_str(), XmlStringStd.length());
 
-	ostringstream output;
-	while(xml.read())
+	try
 	{
-		if(xml.get_depth() == 2 && xml.get_node_type() == xmlpp::TextReader::Element)
-			output << endl << "\t" << xml.get_local_name() << ":\t\t" << xml.read_string();
+		xmlpp::TextReader xml((const unsigned char*)XmlStringStd.c_str(), XmlStringStd.length());
+
+		ostringstream output;
+		while(xml.read())
+		{
+			if(xml.get_depth() == 2 && xml.get_node_type() == xmlpp::TextReader::Element)
+			{
+				output << endl << "\t" << xml.get_local_name() << ":\t\t" << xml.read_string();
+			}
+		}
+		LOG(LOG_INFO, "SWF Metadata:" << output.str());
+	}
+	catch(std::exception& e)
+	{
+		LOG(LOG_INFO, "SWF Metadata:" << XmlStringStd);
 	}
-	LOG(LOG_INFO, "SWF Metadata:" << output.str());
 }
 
 DefineBitsTag::DefineBitsTag(RECORDHEADER h, std::istream& in):DictionaryTag(h)
@@ -1388,8 +1408,7 @@ DefineBitsJPEG2Tag::DefineBitsJPEG2Tag(RECORDHEADER h, std::istream& in):Diction
 	uint8_t* inData=new(nothrow) uint8_t[dataSize];
 	in.read((char*)inData,dataSize);
 
-	bitmapData->fromJPEG(inData,dataSize);
-	Bitmap::updatedData();
+	fromJPEG(inData,dataSize);
 	delete[] inData;
 }
 
@@ -1399,7 +1418,7 @@ ASObject* DefineBitsJPEG2Tag::instance() const
 	if(bindedTo)
 		ret->setClass(bindedTo);
 	else
-		ret->setClass(Class<Bitmap>::getClass());
+		ret->setClass(Class<BitmapData>::getClass());
 	return ret;
 }
 
@@ -1413,8 +1432,7 @@ DefineBitsJPEG3Tag::DefineBitsJPEG3Tag(RECORDHEADER h, std::istream& in):Diction
 	in.read((char*)inData,dataSize);
 
 	//TODO: check header. Could also be PNG or GIF
-	bitmapData->fromJPEG(inData,dataSize);
-	Bitmap::updatedData();
+	fromJPEG(inData,dataSize);
 	delete[] inData;
 
 	//Read alpha data (if any)
@@ -1433,7 +1451,7 @@ ASObject* DefineBitsJPEG3Tag::instance() const
 	if(bindedTo)
 		ret->setClass(bindedTo);
 	else
-		ret->setClass(Class<Bitmap>::getClass());
+		ret->setClass(Class<BitmapData>::getClass());
 	return ret;
 }
 
