@@ -636,6 +636,43 @@ ASObject* ABCVm::executeFunction(const SyntheticFunction* function, call_context
 				context->runtime_stack_push(abstract_b(ret));
 				break;
 			}
+			//Alchemy opcodes
+			case 0x35:
+			{
+				//li8
+				loadIntN<uint8_t>(context);
+				break;
+			}
+			case 0x36:
+			{
+				//li16
+				loadIntN<uint16_t>(context);
+				break;
+			}
+			case 0x37:
+			{
+				//li32
+				loadIntN<uint32_t>(context);
+				break;
+			}
+			case 0x3a:
+			{
+				//si8
+				storeIntN<uint8_t>(context);
+				break;
+			}
+			case 0x3b:
+			{
+				//si16
+				storeIntN<uint16_t>(context);
+				break;
+			}
+			case 0x3c:
+			{
+				//si32
+				storeIntN<uint32_t>(context);
+				break;
+			}
 			case 0x40:
 			{
 				//newfunction
