@@ -22,6 +22,7 @@
 
 #include "compat.h"
 #include "asobject.h"
+#include "../utils/flashutils.h"
 
 namespace lightspark
 {
@@ -32,21 +33,51 @@ public:
 	DLL_PUBLIC static const char* EMULATED_VERSION;
 	Capabilities(){};
 	static void sinit(Class_base* c);
-	ASFUNCTION(_constructor);
 	ASFUNCTION(_getLanguage);
-	ASFUNCTION(playerType);
+	ASFUNCTION(_getPlayerType);
+	ASFUNCTION(_getCPUArchitecture);
+	ASFUNCTION(_getIsDebugger);
+	ASFUNCTION(_getIsEmbeddedInAcrobat);
+	ASFUNCTION(_getLocalFileReadDisable);
+	ASFUNCTION(_getOS);
+	ASFUNCTION(_getVersion);
+	ASFUNCTION(_getServerString);
 };
 
 class ApplicationDomain: public ASObject
 {
 public:
 	ApplicationDomain(){}
+	void finalize();
 	static void sinit(Class_base* c);
 	static void buildTraits(ASObject* o);
 	ASFUNCTION(_constructor);
 	ASFUNCTION(_getCurrentDomain);
 	ASFUNCTION(hasDefinition);
 	ASFUNCTION(getDefinition);
+	ASPROPERTY_GETTER_SETTER(_NR<ByteArray>, domainMemory);
+	template<class T>
+	T readFromDomainMemory(uint32_t addr) const
+	{
+		if(domainMemory.isNull())
+			throw RunTimeException("No memory domain is set");
+		uint32_t bufLen=domainMemory->getLength();
+		if(bufLen < (addr+sizeof(T)))
+			throw RunTimeException("Memory domain access is out of bounds");
+		uint8_t* buf=domainMemory->getBuffer(bufLen, false);
+		return *reinterpret_cast<T*>(buf+addr);
+	}
+	template<class T>
+	void writeToDomainMemory(uint32_t addr, T val)
+	{
+		if(domainMemory.isNull())
+			throw RunTimeException("No memory domain is set");
+		uint32_t bufLen=domainMemory->getLength();
+		if(bufLen < (addr+sizeof(T)))
+			throw RunTimeException("Memory domain access is out of bounds");
+		uint8_t* buf=domainMemory->getBuffer(bufLen, false);
+		*reinterpret_cast<T*>(buf+addr)=val;
+	}
 };
 
 class SecurityDomain: public ASObject
