@@ -32,47 +32,99 @@ namespace lightspark
 {
 const tiny_string flash_proxy="http://www.adobe.com/2006/actionscript/flash/proxy";
 
-class ByteArray: public ASObject
+class Endian : public ASObject
+{
+public:
+	static const char* bigEndian;
+	static const char* littleEndian;
+	Endian(){};
+	static void sinit(Class_base* c);
+};
+
+class IDataInput
+{
+public:
+	static void linkTraits(Class_base* c);
+};
+
+class IDataOutput
+{
+public:
+	static void linkTraits(Class_base* c);
+};
+
+class ByteArray: public ASObject, public IDataInput, public IDataOutput
 {
 friend class Loader;
 friend class URLLoader;
 protected:
 	uint8_t* bytes;
+	uint32_t real_len;
 	uint32_t len;
 	uint32_t position;
+	bool littleEndian;
+	uint32_t objectEncoding;
 	ByteArray(const ByteArray& b);
+	void compress_zlib();
+	void uncompress_zlib();
 public:
 	ByteArray(uint8_t* b = NULL, uint32_t l = 0);
 	~ByteArray();
 	//Helper interface for serialization
 	bool readByte(uint8_t& b);
-	bool readU29(int32_t& ret);
+	bool readShort(uint16_t& ret);
+	bool readUnsignedInt(uint32_t& ret);
+	bool readU29(uint32_t& ret);
+	bool readUTF(tiny_string& ret);
 	void writeByte(uint8_t b);
+	void writeShort(uint16_t val);
+	void writeUnsignedInt(uint32_t val);
+	void writeUTF(const tiny_string& str);
+	uint32_t writeObject(ASObject* obj);
 	void writeStringVR(std::map<tiny_string, uint32_t>& stringMap, const tiny_string& s);
-	void writeU29(int32_t val);
+	void writeU29(uint32_t val);
+	uint32_t getPosition() const;
+	void setPosition(uint32_t p);
 	ASFUNCTION(_getBytesAvailable);
 	ASFUNCTION(_getLength);
 	ASFUNCTION(_setLength);
 	ASFUNCTION(_getPosition);
 	ASFUNCTION(_setPosition);
+	ASFUNCTION(_getEndian);
+	ASFUNCTION(_setEndian);
+	ASFUNCTION(_getObjectEncoding);
+	ASFUNCTION(_setObjectEncoding);
 	ASFUNCTION(_getDefaultObjectEncoding);
 	ASFUNCTION(_setDefaultObjectEncoding);
+	ASFUNCTION(_compress);
+	ASFUNCTION(_uncompress);
+	ASFUNCTION(_deflate);
+	ASFUNCTION(_inflate);
+	ASFUNCTION(clear);
+	ASFUNCTION(readBoolean);
 	ASFUNCTION(readByte);
 	ASFUNCTION(readBytes);
 	ASFUNCTION(readDouble);
 	ASFUNCTION(readFloat);
 	ASFUNCTION(readInt);
-	ASFUNCTION(readUnsignedInt);
+	ASFUNCTION(readMultiByte);
 	ASFUNCTION(readObject);
+	ASFUNCTION(readShort);
+	ASFUNCTION(readUnsignedByte);
+	ASFUNCTION(readUnsignedInt);
+	ASFUNCTION(readUnsignedShort);
 	ASFUNCTION(readUTF);
 	ASFUNCTION(readUTFBytes);
+	ASFUNCTION(writeBoolean);
 	ASFUNCTION(writeByte);
 	ASFUNCTION(writeBytes);
 	ASFUNCTION(writeDouble);
 	ASFUNCTION(writeFloat);
 	ASFUNCTION(writeInt);
 	ASFUNCTION(writeUnsignedInt);
+	ASFUNCTION(writeMultiByte);
 	ASFUNCTION(writeObject);
+	ASFUNCTION(writeShort);
 	ASFUNCTION(writeUTF);
 	ASFUNCTION(writeUTFBytes);
 	ASFUNCTION(_toString);
@@ -87,6 +139,14 @@ public:
 	uint8_t* getBuffer(unsigned int size, bool enableResize);
 	uint32_t getLength() const { return len; }
 
+	uint16_t endianIn(uint16_t value);
+	uint32_t endianIn(uint32_t value);
+	uint64_t endianIn(uint64_t value);
+
+	uint16_t endianOut(uint16_t value);
+	uint32_t endianOut(uint32_t value);
+	uint64_t endianOut(uint64_t value);
+
 	static void sinit(Class_base* c);
 	static void buildTraits(ASObject* o);
 	_NR<ASObject> getVariableByMultiname(const multiname& name, GET_VARIABLE_OPTION opt=NONE);
@@ -165,11 +225,8 @@ public:
 	{
 		setVariableByMultiname(name,abstract_i(value));
 	}
-	bool deleteVariableByMultiname(const multiname& name)
-	{
-		assert_and_throw(implEnable);
-		throw UnsupportedException("deleteVariableByMultiName not supported for Proxy");
-	}
+	
+	bool deleteVariableByMultiname(const multiname& name);
 	bool hasPropertyByMultiname(const multiname& name, bool considerDynamic);
 	tiny_string toString()
 	{
