@@ -30,12 +30,14 @@ class ASError: public ASObject
 CLASSBUILDABLE(ASError);
 protected:
 	ASPROPERTY_GETTER_SETTER(tiny_string, message);
+	void setErrorID(int32_t id) { errorID=id; }
 private:
 	ASPROPERTY_GETTER(int32_t, errorID);
 	ASPROPERTY_GETTER_SETTER(tiny_string, name);
 public:
 	ASError(const tiny_string& error_message = "", int id = 0, const tiny_string& error_name="Error") : message(error_message), errorID(id), name(error_name){};
 	ASFUNCTION(_constructor);
+	ASFUNCTION(generator);
 	ASFUNCTION(getStackTrace);
 	ASFUNCTION(_toString);
 	tiny_string toString(bool debugMsg=false);
@@ -149,8 +151,8 @@ class UninitializedError: public ASError
 CLASSBUILDABLE(UninitializedError);
 	UninitializedError(const tiny_string& error_message = "", int id = 0) : ASError(error_message, id, "UninitializedError"){}
 	ASFUNCTION(_constructor);
-	static void sinit(Class_base* c) {}
-	static void buildTraits(ASObject* o) {}
+	static void sinit(Class_base* c);
+	static void buildTraits(ASObject* o);
 };
 
 }
