@@ -25,11 +25,10 @@
 #include "asobject.h"
 #include "exceptions.h"
 #include "threading.h"
-#include <libxml/tree.h>
-#include <libxml++/parsers/domparser.h>
 #include "scripting/abcutils.h"
 #include "Boolean.h"
 #include "Error.h"
+#include "XML.h"
 
 namespace lightspark
 {
@@ -577,112 +576,6 @@ public:
 			std::map<const ASObject*, uint32_t>& objMap) const;
 };
 
-class XMLList;
-class XML: public ASObject
-{
-friend class XMLList;
-private:
-	//The parser will destroy the document and all the childs on destruction
-	xmlpp::DomParser parser;
-	//Pointer to the root XML element, the one that owns the parser that created this node
-	_NR<XML> root;
-	//The node this object represent
-	xmlpp::Node* node;
-	static void recursiveGetDescendantsByQName(_R<XML> root, xmlpp::Node* node, const tiny_string& name, const tiny_string& ns, 
-			std::vector<_R<XML>>& ret);
-	tiny_string toString_priv();
-	void buildFromString(const std::string& str);
-	bool constructed;
-	bool nodesEqual(xmlpp::Node *a, xmlpp::Node *b) const;
-	XMLList* getAllAttributes();
-	void getText(std::vector<_R<XML>> &ret);
-public:
-	XML();
-	XML(const std::string& str);
-	XML(_R<XML> _r, xmlpp::Node* _n);
-	XML(xmlpp::Node* _n);
-	void finalize();
-	ASFUNCTION(_constructor);
-	ASFUNCTION(_toString);
-	ASFUNCTION(toXMLString);
-	ASFUNCTION(nodeKind);
-	ASFUNCTION(children);
-	ASFUNCTION(attributes);
-	ASFUNCTION(attribute);
-	ASFUNCTION(appendChild);
-	ASFUNCTION(localName);
-	ASFUNCTION(name);
-	ASFUNCTION(descendants);
-	ASFUNCTION(generator);
-	ASFUNCTION(_hasSimpleContent);
-	ASFUNCTION(_hasComplexContent);
-	ASFUNCTION(valueOf);
-	ASFUNCTION(text);
-	static void buildTraits(ASObject* o){};
-	static void sinit(Class_base* c);
-	void getDescendantsByQName(const tiny_string& name, const tiny_string& ns, std::vector<_R<XML> >& ret);
-	_NR<ASObject> getVariableByMultiname(const multiname& name, GET_VARIABLE_OPTION opt);
-	bool hasPropertyByMultiname(const multiname& name, bool considerDynamic);
-	tiny_string toString();
-	void toXMLString_priv(xmlBufferPtr buf);
-	bool hasSimpleContent() const;
-	bool hasComplexContent() const;
-        xmlElementType getNodeKind() const;
-	bool isEqual(ASObject* r);
-	uint32_t nextNameIndex(uint32_t cur_index);
-	_R<ASObject> nextName(uint32_t index);
-	_R<ASObject> nextValue(uint32_t index);
-	//Serialization interface
-	void serialize(ByteArray* out, std::map<tiny_string, uint32_t>& stringMap,
-			std::map<const ASObject*, uint32_t>& objMap) const;
-};
-
-class XMLList: public ASObject
-{
-private:
-	std::vector<_R<XML> > nodes;
-	bool constructed;
-	tiny_string toString_priv() const;
-	void buildFromString(const std::string& str);
-	void toXMLString_priv(xmlBufferPtr buf) const;
-public:
-	XMLList():constructed(false){}
-	/*
-	   Special constructor to build empty XMLList out of AS code
-	*/
-	XMLList(bool c):constructed(c){assert(c);}
-	XMLList(const std::vector<_R<XML> >& r):nodes(r),constructed(true){}
-	XMLList(const std::string& str);
-	void finalize();
-	static void buildTraits(ASObject* o){};
-	static void sinit(Class_base* c);
-	ASFUNCTION(_constructor);
-	ASFUNCTION(_getLength);
-	ASFUNCTION(appendChild);
-	ASFUNCTION(_hasSimpleContent);
-	ASFUNCTION(_hasComplexContent);
-	ASFUNCTION(_toString);
-	ASFUNCTION(toXMLString);
-	ASFUNCTION(generator);
-	ASFUNCTION(descendants);
-	ASFUNCTION(valueOf);
-	ASFUNCTION(text);
-	_NR<ASObject> getVariableByMultiname(const multiname& name, GET_VARIABLE_OPTION opt);
-	void setVariableByMultiname(const multiname& name, ASObject* o);
-	bool hasPropertyByMultiname(const multiname& name, bool considerDynamic);
-	void getDescendantsByQName(const tiny_string& name, const tiny_string& ns, std::vector<_R<XML> >& ret);
-	_NR<XML> convertToXML() const;
-	bool hasSimpleContent() const;
-	bool hasComplexContent() const;
-	void append(_R<XML> x);
-	void append(_R<XMLList> x);
-	tiny_string toString();
-	bool isEqual(ASObject* r);
-	uint32_t nextNameIndex(uint32_t cur_index);
-	_R<ASObject> nextName(uint32_t index);
-	_R<ASObject> nextValue(uint32_t index);
-};
-
 class RegExp: public ASObject
 {
 CLASSBUILDABLE(RegExp);
@@ -731,6 +624,7 @@ public:
 	~GlobalObject();
 };
 
+ASObject* eval(ASObject* obj,ASObject* const* args, const unsigned int argslen);
 ASObject* parseInt(ASObject* obj,ASObject* const* args, const unsigned int argslen);
 ASObject* parseFloat(ASObject* obj,ASObject* const* args, const unsigned int argslen);
 ASObject* isNaN(ASObject* obj,ASObject* const* args, const unsigned int argslen);
